@@ -1,19 +1,20 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2012-2018 Snowflake Computing Inc. All right reserved.
+# Copyright (c) 2012-2019 Snowflake Computing Inc. All right reserved.
 #
 
 
 import base64
-
-from logging import getLogger
 import json
-from Crypto.Cipher import AES
 import os
 import tempfile
 from collections import namedtuple
-from .compat import (PKCS5_PAD, PKCS5_UNPAD, PKCS5_OFFSET, TO_UNICODE)
+from logging import getLogger
+
+from Cryptodome.Cipher import AES
+
+from .compat import PKCS5_OFFSET, PKCS5_PAD, PKCS5_UNPAD, TO_UNICODE
 from .constants import UTF8
 
 
@@ -146,7 +147,7 @@ class SnowflakeEncryptionUtil(object):
             prefix=os.path.basename(in_filename) + "#")
         total_file_size = 0
         prev_chunk = None
-        logger.info(u'encrypted file: %s, tmp file: %s',
+        logger.debug(u'encrypted file: %s, tmp file: %s',
                     in_filename, temp_output_file)
         with open(in_filename, u'rb') as infile:
             with os.fdopen(temp_output_fd, u'wb') as outfile:
