@@ -13,30 +13,30 @@ import tempfile
 import threading
 from logging import getLogger
 from multiprocessing.pool import ThreadPool
-from time import (time, sleep)
+from time import sleep, time
 
 import botocore.exceptions
 
 from .azure_util import SnowflakeAzureUtil
-from .compat import (GET_CWD, TO_UNICODE, IS_WINDOWS)
-from .constants import (SHA256_DIGEST, ResultStatus)
+from .compat import GET_CWD, IS_WINDOWS, TO_UNICODE
+from .constants import SHA256_DIGEST, ResultStatus
 from .converter_snowsql import SnowflakeConverterSnowSQL
-from .errorcode import (ER_INVALID_STAGE_FS, ER_INVALID_STAGE_LOCATION,
-                        ER_LOCAL_PATH_NOT_DIRECTORY,
-                        ER_FILE_NOT_EXISTS,
-                        ER_COMPRESSION_NOT_SUPPORTED,
-                        ER_INTERNAL_NOT_MATCH_ENCRYPT_MATERIAL,
-                        ER_FAILED_TO_DOWNLOAD_FROM_STAGE,
-                        ER_FAILED_TO_UPLOAD_TO_STAGE)
-from .errors import (Error, OperationalError, InternalError, DatabaseError,
-                     ProgrammingError)
+from .errorcode import (
+    ER_COMPRESSION_NOT_SUPPORTED,
+    ER_FAILED_TO_DOWNLOAD_FROM_STAGE,
+    ER_FAILED_TO_UPLOAD_TO_STAGE,
+    ER_FILE_NOT_EXISTS,
+    ER_INTERNAL_NOT_MATCH_ENCRYPT_MATERIAL,
+    ER_INVALID_STAGE_FS,
+    ER_INVALID_STAGE_LOCATION,
+    ER_LOCAL_PATH_NOT_DIRECTORY,
+)
+from .errors import DatabaseError, Error, InternalError, OperationalError, ProgrammingError
 from .file_compression_type import FileCompressionType
 from .file_util import SnowflakeFileUtil
 from .gcs_util import SnowflakeGCSUtil
 from .local_util import SnowflakeLocalUtil
-from .remote_storage_util import (SnowflakeFileEncryptionMaterial,
-                                  SnowflakeRemoteStorageUtil,
-                                  )
+from .remote_storage_util import SnowflakeFileEncryptionMaterial, SnowflakeRemoteStorageUtil
 from .s3_util import SnowflakeS3Util
 
 S3_FS = u'S3'
@@ -323,7 +323,7 @@ class SnowflakeFileTransferAgent(object):
                 len_file_metas
 
             logger.debug(
-                u'uploading files idx: {0}/{1}'.format(idx + 1, end_of_idx))
+                u'uploading files idx: {}/{}'.format(idx + 1, end_of_idx))
 
             target_meta = file_metas[idx:end_of_idx]
             while True:
@@ -337,8 +337,10 @@ class SnowflakeFileTransferAgent(object):
                 # need renew AWS token?
                 retry_meta = []
                 for result_meta in results:
-                    if result_meta[
-                        u'result_status'] == ResultStatus.RENEW_TOKEN:
+                    if result_meta[u'result_status'] in [
+                        ResultStatus.RENEW_TOKEN,
+                        ResultStatus.RENEW_PRESIGNED_URL
+                    ]:
                         retry_meta.append(result_meta)
                     else:
                         self._results.append(result_meta)
@@ -346,9 +348,14 @@ class SnowflakeFileTransferAgent(object):
                 if len(retry_meta) == 0:
                     # no new AWS token is required
                     break
-                client = self.renew_expired_client()
-                for result_meta in retry_meta:
-                    result_meta[u'client'] = client
+                if any([result_meta[u'result_status'] == ResultStatus.RENEW_TOKEN
+                        for result_meta in results]):
+                    client = self.renew_expired_client()
+                    for result_meta in retry_meta:
+                        result_meta[u'client'] = client
+                if any([result_meta[u'result_status'] == ResultStatus.RENEW_PRESIGNED_URL
+                        for result_meta in results]):
+                    self._update_file_metas_with_presigned_url()
                 if end_of_idx < len_file_metas:
                     for idx0 in range(idx + self._parallel, len_file_metas):
                         file_metas[idx0][u'client'] = client
@@ -366,7 +373,7 @@ class SnowflakeFileTransferAgent(object):
         len_file_metas = len(file_metas)
         while idx < len_file_metas:
             logger.debug(
-                u'uploading files idx: {0}/{1}'.format(idx + 1, len_file_metas))
+                u'uploading files idx: {}/{}'.format(idx + 1, len_file_metas))
             result = SnowflakeFileTransferAgent.upload_one_file(
                 file_metas[idx])
             if result[u'result_status'] == ResultStatus.RENEW_TOKEN:
@@ -374,6 +381,9 @@ class SnowflakeFileTransferAgent(object):
                 for idx0 in range(idx, len_file_metas):
                     file_metas[idx0][u'client'] = client
                 continue
+            elif result[u'result_status'] == ResultStatus.RENEW_PRESIGNED_URL:
+                self._update_file_metas_with_presigned_url()
+                continue
             self._results.append(result)
             idx += 1
             if INJECT_WAIT_IN_PUT > 0:
@@ -469,7 +479,7 @@ class SnowflakeFileTransferAgent(object):
                 len_file_metas
 
             logger.debug(
-                'downloading files idx: {0} to {1}'.format(idx, end_of_idx))
+                'downloading files idx: {} to {}'.format(idx, end_of_idx))
 
             target_meta = file_metas[idx:end_of_idx]
             while True:
@@ -484,7 +494,10 @@ class SnowflakeFileTransferAgent(object):
                 retry_meta = []
                 for result_meta in results:
                     if result_meta[
-                        u'result_status'] == ResultStatus.RENEW_TOKEN:
+                        u'result_status'] in [
+                        ResultStatus.RENEW_TOKEN,
+                        ResultStatus.RENEW_PRESIGNED_URL
+                    ]:
                         retry_meta.append(result_meta)
                     else:
                         self._results.append(result_meta)
@@ -492,9 +505,14 @@ class SnowflakeFileTransferAgent(object):
                 if len(retry_meta) == 0:
                     # no new AWS token is required
                     break
-                client = self.renew_expired_client()
-                for result_meta in retry_meta:
-                    result_meta[u'client'] = client
+                if any([result_meta[u'result_status'] == ResultStatus.RENEW_TOKEN
+                        for result_meta in results]):
+                    client = self.renew_expired_client()
+                    for result_meta in retry_meta:
+                        result_meta[u'client'] = client
+                if any([result_meta[u'result_status'] == ResultStatus.RENEW_PRESIGNED_URL
+                        for result_meta in results]):
+                    self._update_file_metas_with_presigned_url()
                 if end_of_idx < len_file_metas:
                     for idx0 in range(idx + self._parallel, len_file_metas):
                         file_metas[idx0][u'client'] = client
@@ -518,6 +536,9 @@ class SnowflakeFileTransferAgent(object):
                 for idx0 in range(idx, len_file_metas):
                     file_metas[idx0][u'client'] = client
                 continue
+            elif result[u'result_status'] == ResultStatus.RENEW_PRESIGNED_URL:
+                self._update_file_metas_with_presigned_url()
+                continue
             self._results.append(result)
             idx += 1
             if INJECT_WAIT_IN_PUT > 0:
@@ -621,8 +642,7 @@ class SnowflakeFileTransferAgent(object):
 
                     ret = self._cursor._execute_helper(command_with_single_file)
 
-                    if ret.get(u'data', None) and \
-                            ret[u'data'].get(u'stageInfo', None):
+                    if ret.get(u'data', dict()).get(u'stageInfo', None):
                         meta[u'stage_info'] = ret[u'data'][u'stageInfo']
                         meta[u'presigned_url'] = meta[u'stage_info'].get(
                             u'presignedUrl', None)
@@ -844,7 +864,7 @@ class SnowflakeFileTransferAgent(object):
                     ProgrammingError,
                     {
                         u'msg':
-                            u'The local path is not a directory: {0}'.format(
+                            u'The local path is not a directory: {}'.format(
                                 self._local_location),
                         u'errno': ER_LOCAL_PATH_NOT_DIRECTORY
                     })
