@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2012-2018 Snowflake Computing Inc. All right reserved.
+# Copyright (c) 2012-2019 Snowflake Computing Inc. All right reserved.
 #
 
 from __future__ import division
@@ -12,11 +12,11 @@ import time
 from collections import namedtuple
 from logging import getLogger
 
-from .s3_util import SnowflakeS3Util
 from .azure_util import SnowflakeAzureUtil
-
-from .constants import (SHA256_DIGEST, ResultStatus)
-from .encryption_util import (SnowflakeEncryptionUtil)
+from .constants import ResultStatus
+from .encryption_util import SnowflakeEncryptionUtil
+from .gcs_util import SnowflakeGCSUtil
+from .s3_util import SnowflakeS3Util
 
 DEFAULT_CONCURRENCY = 1
 DEFAULT_MAX_RETRY = 5
@@ -40,19 +40,26 @@ class NeedRenewTokenError(Exception):
 class SnowflakeRemoteStorageUtil(object):
 
     @staticmethod
-    def getStorageType(type):
-        if(type == u'S3'):
+    def getForStorageType(type):
+        if (type == u'S3'):
             return SnowflakeS3Util
-        elif(type == u'AZURE'):
+        elif (type == u'AZURE'):
             return SnowflakeAzureUtil
+        elif (type == u'GCS'):
+            return SnowflakeGCSUtil
+        else:
+            return None
 
     @staticmethod
     def create_client(stage_info, use_accelerate_endpoint=False):
-        util_class = SnowflakeRemoteStorageUtil.getStorageType(stage_info[u'locationType'])
-        return util_class.create_client(stage_info, use_accelerate_endpoint=False)
+        util_class = SnowflakeRemoteStorageUtil.getForStorageType(
+            stage_info[u'locationType'])
+        return util_class.create_client(
+            stage_info,
+            use_accelerate_endpoint=use_accelerate_endpoint)
 
     @staticmethod
-    def upload_one_file_to_s3(meta):
+    def upload_one_file(meta):
         """
         Uploads a file to S3
         :param meta: a file meta
@@ -61,7 +68,8 @@ class SnowflakeRemoteStorageUtil(object):
         encryption_metadata = None
 
         if u'encryption_material' in meta:
-            (encryption_metadata, data_file) = SnowflakeEncryptionUtil.encrypt_file(
+            (encryption_metadata,
+             data_file) = SnowflakeEncryptionUtil.encrypt_file(
                 meta[u'encryption_material'],
                 meta[u'real_src_file_name'], tmp_dir=meta[u'tmp_dir'])
 
@@ -71,32 +79,29 @@ class SnowflakeRemoteStorageUtil(object):
         else:
             logger.debug(u'not encrypted data file')
             data_file = meta[u'real_src_file_name']
-        util_class = SnowflakeRemoteStorageUtil.getStorageType(meta[u'stage_info'][u'locationType'])
-        file_header = util_class.get_file_header(meta, meta[u'dst_file_name'])
-        if meta[u'result_status'] == ResultStatus.RENEW_TOKEN:
-            # need renew token
-            return
-        elif file_header and meta[u'result_status'] == ResultStatus.UPLOADED and \
-                not meta.get(u'overwrite'):
-            logger.info(
-                u'file already exists, checking digest: location="%s", file_name="%s"', meta[u'stage_info'][u'location'], meta[u'dst_file_name'])
-            sfc_digest = file_header.digest
-            if sfc_digest == meta[SHA256_DIGEST]:
-                logger.info(u'file digest matched: digest=%s',
-                            sfc_digest)
+
+        util_class = SnowflakeRemoteStorageUtil.getForStorageType(
+            meta[u'stage_info'][u'locationType'])
+        if not meta.get(u'overwrite'):
+            file_header = util_class.get_file_header(
+                meta, meta[u'dst_file_name'])
+            if meta[u'result_status'] == ResultStatus.RENEW_TOKEN:
+                # need renew token
+                return
+            elif meta[u'result_status'] == ResultStatus.RENEW_PRESIGNED_URL:
+                return
+            elif file_header and \
+                    meta[u'result_status'] == ResultStatus.UPLOADED:
+                logger.debug(
+                    u'file already exists location="%s", file_name="%s"',
+                    meta[u'stage_info'][u'location'],
+                    meta[u'dst_file_name'])
                 meta[u'dst_file_size'] = 0
-                meta[u'error_details'] = \
-                    (u'File with the same destination name '
-                     u'and checksum already exists')
                 meta[u'result_status'] = ResultStatus.SKIPPED
                 return
-            else:
-                logger.info(
-                    (u"file digest didn't match: "
-                     u"digest_s3=%s, digest_local=%s"),
-                    sfc_digest, meta[SHA256_DIGEST])
 
-        logger.debug(u'putting a file: %s, %s', meta[u'stage_info'][u'location'], meta[u'dst_file_name'])
+        logger.debug(u'putting a file: %s, %s',
+                     meta[u'stage_info'][u'location'], meta[u'dst_file_name'])
 
         max_concurrency = meta[u'parallel']
         last_err = None
@@ -109,13 +114,15 @@ class SnowflakeRemoteStorageUtil(object):
                 max_concurrency
             )
 
-            if(meta[u'result_status'] == ResultStatus.UPLOADED):
+            if (meta[u'result_status'] == ResultStatus.UPLOADED):
                 return
             elif (meta[u'result_status'] == ResultStatus.RENEW_TOKEN):
                 return
-            elif(meta[u'result_status'] == ResultStatus.NEED_RETRY):
+            elif (meta[u'result_status'] == ResultStatus.RENEW_PRESIGNED_URL):
+                return
+            elif (meta[u'result_status'] == ResultStatus.NEED_RETRY):
                 last_err = meta[u'last_error']
-                logger.info(
+                logger.debug(
                     'Failed to upload a file: %s, err: %s. Retrying with '
                     'max concurrency: %s',
                     data_file, last_err, max_concurrency)
@@ -123,14 +130,15 @@ class SnowflakeRemoteStorageUtil(object):
                     sleeping_time = min(2 ** retry, 16)
                     logger.debug(u"sleeping: %s", sleeping_time)
                     time.sleep(sleeping_time)
-            elif(meta[u'result_status'] == ResultStatus.NEED_RETRY_WITH_LOWER_CONCURRENCY):
+            elif (meta[
+                      u'result_status'] == ResultStatus.NEED_RETRY_WITH_LOWER_CONCURRENCY):
                 last_err = meta[u'last_error']
                 max_concurrency = meta[u'parallel'] - int(
                     retry * meta[u'parallel'] / max_retry)
                 max_concurrency = max(DEFAULT_CONCURRENCY, max_concurrency)
                 meta['last_max_concurrency'] = max_concurrency
 
-                logger.info(
+                logger.debug(
                     'Failed to upload a file: %s, err: %s. Retrying with '
                     'max concurrency: %s',
                     data_file, last_err, max_concurrency)
@@ -162,9 +170,12 @@ class SnowflakeRemoteStorageUtil(object):
         if not os.path.exists(base_dir):
             os.makedirs(base_dir)
 
-        util_class = SnowflakeRemoteStorageUtil.getStorageType(meta[u'stage_info'][u'locationType'])
+        util_class = SnowflakeRemoteStorageUtil.getForStorageType(
+            meta[u'stage_info'][u'locationType'])
         file_header = util_class.get_file_header(meta, meta[u'src_file_name'])
-        meta[u'src_file_size'] = file_header.content_length
+
+        if file_header:
+            meta[u'src_file_size'] = file_header.content_length
 
         full_dst_file_name = os.path.join(
             meta[u'local_location'],
@@ -175,13 +186,28 @@ class SnowflakeRemoteStorageUtil(object):
         last_err = None
         max_retry = DEFAULT_MAX_RETRY
         for retry in range(max_retry):
-            util_class._native_download_file(meta, full_dst_file_name, max_concurrency)
-            if(meta[u'result_status'] == ResultStatus.DOWNLOADED):
+            util_class._native_download_file(meta, full_dst_file_name,
+                                             max_concurrency)
+            if (meta[u'result_status'] == ResultStatus.DOWNLOADED):
                 if u'encryption_material' in meta:
                     logger.debug(
                         u'encrypted data file=%s', full_dst_file_name)
+
+                    # For storage utils that do not have the privilege of
+                    # getting the metadata early, both object and metadata
+                    # are downloaded at once. In which case, the file meta will
+                    # be updated with all the metadata that we need and
+                    # then we can call get_file_header to get just that and also
+                    # preserve the idea of getting metadata in the first place.
+                    # One example of this is the utils that use presigned url
+                    # for upload/download and not the storage client library.
+                    if meta.get(u'presigned_url', None):
+                        file_header = util_class.get_file_header(meta, meta[
+                            u'src_file_name'])
+
                     tmp_dst_file_name = SnowflakeEncryptionUtil.decrypt_file(
-                        file_header.encryption_metadata, meta[u'encryption_material'],
+                        file_header.encryption_metadata,
+                        meta[u'encryption_material'],
                         full_dst_file_name, tmp_dir=meta[u'tmp_dir'])
                     shutil.copyfile(tmp_dst_file_name, full_dst_file_name)
                     os.unlink(tmp_dst_file_name)
@@ -192,9 +218,12 @@ class SnowflakeRemoteStorageUtil(object):
                 statinfo = os.stat(full_dst_file_name)
                 meta[u'dst_file_size'] = statinfo.st_size
                 return
+            elif (meta[u'result_status'] == ResultStatus.RENEW_PRESIGNED_URL):
+                return
             elif (meta[u'result_status'] == ResultStatus.RENEW_TOKEN):
                 return
-            elif (meta[u'result_status'] == ResultStatus.NEED_RETRY_WITH_LOWER_CONCURRENCY):
+            elif (meta[
+                      u'result_status'] == ResultStatus.NEED_RETRY_WITH_LOWER_CONCURRENCY):
                 max_concurrency = meta[u'parallel'] - int(
                     retry * meta[u'parallel'] / max_retry)
                 max_concurrency = max(DEFAULT_CONCURRENCY, max_concurrency)
@@ -208,7 +237,7 @@ class SnowflakeRemoteStorageUtil(object):
                     sleeping_time = min(2 ** retry, 16)
                     logger.debug(u"sleeping: %s", sleeping_time)
                     time.sleep(sleeping_time)
-            elif(meta[u'result_status'] == ResultStatus.NEED_RETRY):
+            elif (meta[u'result_status'] == ResultStatus.NEED_RETRY):
                 last_err = meta[u'last_error']
                 logger.debug(
                     'Failed to download a file: %s, err: %s. Retrying with '
@@ -229,15 +258,16 @@ class SnowflakeRemoteStorageUtil(object):
     @staticmethod
     def upload_one_file_with_retry(meta):
         """
-        Uploads one file to S3 with retry
+        Uploads one file with retry
         :param meta: a file meta
         """
         logger = getLogger(__name__)
 
-        util_class = SnowflakeRemoteStorageUtil.getStorageType(meta[u'stage_info'][u'locationType'])
+        util_class = SnowflakeRemoteStorageUtil.getForStorageType(
+            meta[u'stage_info'][u'locationType'])
         for _ in range(10):
             # retry
-            SnowflakeRemoteStorageUtil.upload_one_file_to_s3(meta)
+            SnowflakeRemoteStorageUtil.upload_one_file(meta)
             if meta[u'result_status'] == ResultStatus.UPLOADED:
                 for _ in range(10):
                     util_class.get_file_header(
