@@ -1,12 +1,15 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2012-2018 Snowflake Computing Inc. All right reserved.
+# Copyright (c) 2012-2019 Snowflake Computing Inc. All right reserved.
 #
 import logging
 import re
 
-import ijson
+try:
+    import ijson
+except Exception:
+    ijson = None
 
 COMMENT_PATTERN_RE = re.compile(r'^\s*\-\-')
 EMPTY_LINE_RE = re.compile(r'^\s*$')
@@ -45,7 +48,7 @@ def split_statements(buf, remove_comments=False):
                         if len(statement) == 1 and statement[0][0] == '':
                             statement = []
                         break
-                    elif not in_comment and in_quote:
+                    elif not in_comment and (in_quote or in_double_dollars):
                         statement.append((line[col0:col], True))
                     elif not remove_comments:
                         statement.append((line[col0:col], False))
@@ -68,9 +71,12 @@ def split_statements(buf, remove_comments=False):
                 else:
                     col += 1
             elif in_quote:
-                if line[col] == ch_quote:
+                if line[col] == '\\' and col < len_line - 1 and \
+                        line[col + 1] in (ch_quote, '\\'):
+                    col += 2
+                elif line[col] == ch_quote:
                     if col < len_line - 1 and line[col + 1] != ch_quote or \
-                                    col == len_line - 1:
+                            col == len_line - 1:
                         # exits quote
                         in_quote = False
                         statement.append((line[col0:col + 1], True))
@@ -101,6 +107,8 @@ def split_statements(buf, remove_comments=False):
                         not line[col0:].startswith("file://"):
                     if not remove_comments:
                         statement.append((line[col0:col + 2], False))
+                    else:
+                        statement.append((line[col0:col], False))
                     col += 2
                     col0 = col
                     in_comment = True
@@ -166,6 +174,8 @@ def split_rows_from_stream(stream):
     """
     Splits into rows from a stream object. Generator.
     """
+    if not ijson:
+        raise Exception("install ijson")
     row = []
     in_row = False
     for prefix, event, value in ijson.parse(stream):
@@ -186,10 +196,29 @@ def construct_hostname(region, account):
     """
     Constructs hostname from region and account
     """
+    if region == u'us-west-2':
+        region = ''
     if region:
         if account.find(u'.') > 0:
             account = account[0:account.find(u'.')]
-        host = u'{0}.{1}.snowflakecomputing.com'.format(account, region)
+        host = u'{}.{}.snowflakecomputing.com'.format(account, region)
     else:
-        host = u'{0}.snowflakecomputing.com'.format(account)
+        host = u'{}.snowflakecomputing.com'.format(account)
     return host
+
+
+def parse_account(account):
+    url_parts = account.split(u'.')
+    # if this condition is true, then we have some extra
+    # stuff in the account field.
+    if len(url_parts) > 1:
+        if url_parts[1] == u'global':
+            # remove external ID from account
+            parsed_account = url_parts[0][0:url_parts[0].rfind(u'-')]
+        else:
+            # remove region subdomain
+            parsed_account = url_parts[0]
+    else:
+        parsed_account = account
+
+    return parsed_account
